//   Copyright 2012 Georg-August-Universität Göttingen, Germany
//
//   Licensed under the Apache License, Version 2.0 (the "License");
//   you may not use this file except in compliance with the License.
//   You may obtain a copy of the License at
//
//       http://www.apache.org/licenses/LICENSE-2.0
//
//   Unless required by applicable law or agreed to in writing, software
//   distributed under the License is distributed on an "AS IS" BASIS,
//   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
//   See the License for the specific language governing permissions and
//   limitations under the License.

package de.ugoe.cs.autoquest.commands.sequences;

import java.text.SimpleDateFormat;
import java.util.Collection;
import java.util.Date;
import java.util.List;
import java.util.Map.Entry;
import java.util.SortedMap;
import java.util.TreeMap;

import de.ugoe.cs.autoquest.CommandHelpers;
import de.ugoe.cs.autoquest.SequenceInstanceOf;
import de.ugoe.cs.autoquest.eventcore.Event;
import de.ugoe.cs.util.console.Command;
import de.ugoe.cs.util.console.Console;
import de.ugoe.cs.util.console.GlobalDataContainer;

/**
 * <p>
 * Command to print basic statistical information about stored sequences, e.g., how many there are
 * of which length.
 * </p>
 * 
 * @author Steffen Herbold
 * @version 1.0
 */
public class CMDsequenceStatistics implements Command {

    /*
     * (non-Javadoc)
     * 
     * @see de.ugoe.cs.util.console.Command#run(java.util.List)
     */
    @SuppressWarnings("unchecked")
    @Override
    public void run(List<Object> parameters) {
        String sequencesName;
        try {
            sequencesName = (String) parameters.get(0);
        }
        catch (Exception e) {
            throw new IllegalArgumentException();
        }

        Collection<List<Event>> sequences = null;
        Object dataObject = GlobalDataContainer.getInstance().getData(sequencesName);
        if (dataObject == null) {
            CommandHelpers.objectNotFoundMessage(sequencesName);
            return;
        }
        if (!SequenceInstanceOf.isCollectionOfSequences(dataObject)) {
            CommandHelpers.objectNotType(sequencesName, "Collection<List<Event<?>>>");
            return;
        }

        sequences = (Collection<List<Event>>) dataObject;
        Console.println("Number of Sequences: " + sequences.size());

        long smallestTimeStamp = Long.MAX_VALUE;
        long largestTimeStamp = 0;
        int noOfEvents = 0;

        SortedMap<Integer, Integer> lengthMap = new TreeMap<Integer, Integer>();
        for (List<Event> sequence : sequences) {
            Integer currentSize = sequence.size();
            noOfEvents += currentSize;
            
            if (lengthMap.containsKey(currentSize)) {
                lengthMap.put(currentSize, lengthMap.get(currentSize) + 1);
            }
            else {
                lengthMap.put(currentSize, 1);
            }
            
            for (Event event : sequence) {
                if (event.getTimestamp() > 0) {
                    smallestTimeStamp = Math.min(smallestTimeStamp, event.getTimestamp());
                    largestTimeStamp = Math.max(largestTimeStamp, event.getTimestamp());
                }
            }
        }

        Console.println("Number of Events: " + noOfEvents);
        Console.println("First Event Recorded At: " +
                        SimpleDateFormat.getDateTimeInstance().format(new Date(smallestTimeStamp)));
        Console.println("Last Event Recorded At: " +
                        SimpleDateFormat.getDateTimeInstance().format(new Date(largestTimeStamp)));
        
        for (Entry<Integer, Integer> entry : lengthMap.entrySet()) {
            Console.println("Sequences of Length " + entry.getKey() + ": " + entry.getValue());
        }
    }

    /*
     * (non-Javadoc)
     * 
     * @see de.ugoe.cs.util.console.Command#help()
     */
    @Override
    public String help() {
        return "sequenceStatistics <sequencesName>";
    }

}
